/*
 * $Id: AbstractConnector.java,v 1.7 2005/10/13 12:37:25 windowsnt\martin Exp $
 * 
 * Copyright (C) ITC 2005 - International Institute for Geo-Information Science and Earth Observation
 *    
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package org.n52.connectors.core;

import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.ArrayList;

import javax.xml.transform.TransformerException;

import org.geotools.metadata.iso.MetadataEntity;
import org.n52.connectors.core.exceptions.N52Exception;
import org.n52.connectors.core.exceptions.WrappedException;
import org.w3c.dom.Document;

/**
 * Contains a number of default implementations of methods of the interface. The
 * class is the mandatory parent for all implementations of the Connector
 * interface
 * 
 * @author martin schouwenburg
 * 
 */
public abstract class AbstractConnector implements Connector {

	/* (non-Javadoc)
	 * @see org.n52.connectors.core.Connector#isSupported(org.n52.connectors.core.ConnectorRequest)
	 */
	public abstract boolean isSupported(ServiceRequest request);

	/* (non-Javadoc)
	 * @see org.n52.connectors.core.Connector#getMetadata(java.lang.String)
	 */
	public abstract MetadataEntity getMetadata(String arg0);

	public String getName() {
		return connectorName;
	}

	protected String connectorName = "";

	private ArrayList<Dependency> dependencies = new ArrayList<Dependency>();

	private String implementingClass;

	private URL classUrl;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.n52.connectors.core.Connector#registrySideInitialization(java.lang.String,
	 *      org.n52.connectors.core.ConnectorRequest) empty implementation;
	 *      maybe overruled by derived connector
	 */
	public String registrySideInitialization(ServiceRequest request) {
		return "";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.n52.connectors.core.Connector#clientSideInitialization(org.w3c.dom.Document,
	 *      org.n52.connectors.core.ConnectorRequest) empty implementation;
	 *      maybe overruled by derived connector
	 */
	public void factorySideInitialization(Document xmlConfiguration,
			ServiceRequest request) throws N52Exception {
		try {
			implementingClass = ServiceFactory.getNodeValue(xmlConfiguration
					.getDocumentElement(), "implementingclass");
			classUrl = new URL(ServiceFactory.getNodeValue(
					xmlConfiguration.getDocumentElement(), "classUrl"));
		} catch (TransformerException e) {
			throw new WrappedException(e);
		} catch (MalformedURLException e) {
			throw new WrappedException(e);
		}
	}

	protected void addDependency(Dependency dependency) {
		if (dependencies.contains(dependency))
			return;

		dependencies.add(dependency);

	}

	/* (non-Javadoc)
	 * @see org.n52.connectors.core.Connector#getImplementingObject(org.n52.connectors.core.ConnectorRequest)
	 */
	public Object getImplementingObject(ServiceRequest request)
			throws N52Exception {
		try {

			URL[] urlsToLoadFrom = new URL[] { classUrl };
			URLClassLoader loader = new URLClassLoader(urlsToLoadFrom);
			Class operationClass = Class.forName(implementingClass, true, loader);
			return operationClass.newInstance();
		} catch (ClassNotFoundException ex) {
			throw new WrappedException(ex);
		} catch (InstantiationException ex) {
			throw new WrappedException(ex);
		} catch (IllegalAccessException ex) {
			throw new WrappedException(ex);
		}

	}

}
