/*
 * $Id: AbstractServiceFactory.java,v 1.1 2005/10/13 12:37:26 windowsnt\martin Exp $
 * 
 * Copyright (C) ITC 2005 - International Institute for Geo-Information Science and Earth Observation
 *    
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package org.n52.connectors.core;

import java.io.IOException;
import java.io.StringReader;
import java.net.URL;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * Adds a number of convenience functions to the ConnectorCreation. It also
 * contains the core of the administration for the available regsitries. Every
 * connector must be told where it can find its registries. Local registries are
 * added by setting a locally created registry object; remote registries can be
 * added by setting an URL.
 * 
 * @author martin schouwenburg
 */
public abstract class AbstractServiceFactory implements ConnectorCreation {

	public void addRegistry(ConnectorTypeRegistry registry) {
		registries.add(registry);

	}

	public void addRegistry(URL location) {
		registries.add(new RemoteConnectorTypeRegistryProxy(location));

	}

	protected Document getClassInfo(String respons) throws SAXException,
			IOException, TransformerException, ParserConfigurationException {

		StringReader reader = new StringReader(respons);
		InputSource source = new InputSource(reader);
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder build = factory.newDocumentBuilder();
		return build.parse(source);
	}

	protected ArrayList<ConnectorTypeRegistry> registries = new ArrayList<ConnectorTypeRegistry>();
	
}
