/*
 * $Id: ConnectorTypeRegistry.java,v 1.7 2005/10/13 12:37:27 windowsnt\martin Exp $
 * 
 * Copyright (C) ITC 2005 - International Institute for Geo-Information Science and Earth Observation
 *    
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package org.n52.connectors.core;

import org.n52.connectors.core.exceptions.IllegalConnectorRequestException;
import org.n52.connectors.core.exceptions.ParameterDoesNotExistException;
import org.n52.connectors.core.exceptions.WrappedException;

/**
 * The interface to support the creation of the necessary information for the
 * factory to load a connector class. The registry itself does not create/load
 * connectors
 * 
 * @author martin schouwenburg
 * 
 */
public interface ConnectorTypeRegistry {

	/**
	 * return an XML String describing a Connector with requested functionality.
	 * Depending on the request, the connector returned is either the default
	 * for the service or the connector explicitly requested for. In the XML
	 * string all needed information should be that enables a factory to create
	 * a proper connector with the correct initialization
	 * 
	 * @param request
	 *            contains all information that is needed to find a suitable
	 *            connector
	 * @return a connector that supports the request, in an XML String
	 * @throws IllegalConnectorRequestException
	 *             request could not be fullfilled
	 * @throws WrappedException
	 * @throws ParameterDoesNotExistException
	 */
	String getConnectorClass(ServiceRequest request)
			throws IllegalConnectorRequestException, WrappedException,
			ParameterDoesNotExistException;
}
