/*
 * $Id: ServiceFactory.java,v 1.1 2005/10/13 12:37:28 windowsnt\martin Exp $
 * 
 * Copyright (C) ITC 2005 - International Institute for Geo-Information Science and Earth Observation
 *    
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package org.n52.connectors.core;

import java.io.IOException;
import java.net.URL;
import java.net.URLClassLoader;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import org.n52.connectors.core.exceptions.IllegalConnectorRequestException;
import org.n52.connectors.core.exceptions.N52Exception;
import org.n52.connectors.core.exceptions.WrappedException;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

import com.sun.org.apache.xpath.internal.XPathAPI;

/**
 * Implements the abstract factory. The factory is the main interface for
 * non-applet applications to access functionality provided by the connector
 * system {@see AbstractFactory}. A user of a factory formulates a request and
 * will get back a connector which supports the request.
 * 
 * @author martin schouwenburg
 * 
 */
public class ServiceFactory extends AbstractServiceFactory {

	/**
	 * Retrieves a connector that can retrieve and initialize an object that
	 * supports the requested functionality
	 * 
	 * @param request
	 * @return
	 * @throws N52Exception
	 */
	Connector getConnector(ServiceRequest request) throws N52Exception {
		if (registries == null)
			throw new IllegalConnectorRequestException(
					"No registries defined, request can not be handled");

		for (ConnectorTypeRegistry registry : registries) {
			try {

				String respons = registry.getConnectorClass(request);
				URL classUrl = new URL("http://empty");
				String className = new String();
				Document doc = getClassInfo(respons);
				className = getNodeValue(doc.getDocumentElement(), "classname");
				classUrl = new URL(getNodeValue(doc.getDocumentElement(),
						"classUrl"));
				if (classUrl.equals(""))
					continue;

				URL[] urlsToLoadFrom = new URL[] { classUrl };
				URLClassLoader loader = new URLClassLoader(urlsToLoadFrom);
				Class connectorClass = Class.forName(className, true, loader);
				Connector conn = (Connector) connectorClass.newInstance();
				conn.factorySideInitialization(doc, request);
				return conn;
			} catch (ClassNotFoundException ex) {
				throw new WrappedException(ex);
			} catch (InstantiationException ex) {
				throw new WrappedException(ex);
			} catch (IllegalAccessException ex) {
				throw new WrappedException(ex);
			} catch (SAXException ex) {
				throw new WrappedException(ex);
			} catch (IOException ex) {
				throw new WrappedException(ex);
			} catch (TransformerException ex) {
				throw new WrappedException(ex);
			} catch (ParserConfigurationException ex) {
				throw new WrappedException(ex);
			}

		}
		return null;
	}

	static public String getNodeValue(Node n, String tag)
			throws TransformerException {
		Node n2 = XPathAPI.selectSingleNode(n, tag + "/text()");
		return n2.getNodeValue();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.n52.connectors.core.ConnectorCreation#getServiceObject(org.n52.connectors.core.ConnectorRequest)
	 */
	public Object getServiceAdapter(ServiceRequest request)
			throws N52Exception {
		Connector conn = getConnector(request);
		return conn.getImplementingObject(request);
	}

	public Object[] getServiceAdapters(ServiceRequest request) {
		// TODO Auto-generated method stub
		return null;
	}

}
