package org.n52.connectors.testcases;

import java.io.IOException;
import java.io.StringReader;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerException;

import junit.framework.TestCase;

import org.n52.connectors.core.AbstractConnectorTypeRegistry;
import org.n52.connectors.core.ServiceRequest;
import org.n52.connectors.core.LocalConnectorTypeRegistry;
import org.n52.connectors.core.exceptions.IllegalConnectorRequestException;
import org.n52.connectors.core.exceptions.IllegalDefaultSettingException;
import org.n52.connectors.core.exceptions.ParameterDoesNotExistException;
import org.n52.connectors.core.exceptions.WrappedException;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import com.sun.org.apache.xerces.internal.impl.xs.dom.DOMParser;
import com.sun.org.apache.xpath.internal.XPathAPI;

public class LocalConnectorTypeRegistryTester extends TestCase {

	public void testGetConnectorClass() {
		try {
			ServiceRequest request = new ServiceRequest();
			request.put("service", "Test");

			AbstractConnectorTypeRegistry actr = new LocalConnectorTypeRegistry(
					"../ConnectorCore");
			actr.setDefaultConnector("Test", "itc.TestConnector"); // force the
			// default
			// to be
			// what we
			// want to
			// test
			String xmlConnectorClass = actr.getConnectorClass(request);
			StringReader reader = new StringReader(xmlConnectorClass);
			InputSource xmlSource = new InputSource(reader);

			DocumentBuilderFactory factory = DocumentBuilderFactory
					.newInstance();
			DocumentBuilder build = factory.newDocumentBuilder();
			Document doc = build.parse(xmlSource);

			Node n = XPathAPI.selectSingleNode(doc.getDocumentElement(),
					"classname/text()");

			String connectorName = n.getNodeValue();

			assertEquals(connectorName, "org.n52.connectors.impl.TestConnector");
		} catch (IllegalConnectorRequestException e) {
			assertTrue(false);
		} catch (SAXException e) {
			assertTrue(false);
		} catch (IOException e) {
			assertTrue(false);
		} catch (TransformerException e) {
			assertTrue(false);
		} catch (ParserConfigurationException e) {
			assertTrue(false);
		} catch (WrappedException e) {
			assertTrue(false);
		} catch (IllegalDefaultSettingException e) {
			assertTrue(false);
		} catch (ParameterDoesNotExistException e) {
			assertTrue(false);
		}
	}

	public void testGetConnectorClassBadRequest() {
		try {
			ServiceRequest request = new ServiceRequest();
			request.put("service", "GS.GradCoverage.Open"); // intentional
			// typo

			AbstractConnectorTypeRegistry actr = new LocalConnectorTypeRegistry(
					"../ConnectorCore");
			actr.setDefaultConnector("GS.GridCoverage.Open",
					"itc.GdalConnector"); // force the default to be what we
			// want to test
			String xmlConnectorClass = actr.getConnectorClass(request);

			StringReader reader = new StringReader(xmlConnectorClass);
			InputSource xmlSource = new InputSource(reader);
			DOMParser dp = new DOMParser();
			dp.parse(xmlSource);
			Document doc = dp.getDocument();
			Node n = XPathAPI.selectSingleNode(doc.getDocumentElement(),
					"*/classname/text()");

			String className = n.getNodeValue();

			assertEquals(className, "org.n52.connectors.impl.GdalConnector");
		} catch (IllegalConnectorRequestException e) {
			assertTrue(true); // the only acceptable case
		} catch (SAXException e) {
			assertTrue(false);
		} catch (IOException e) {
			assertTrue(false);
		} catch (TransformerException e) {
			assertTrue(false);
		} catch (WrappedException e) {
			assertTrue(false);
		} catch (IllegalDefaultSettingException e) {
			assertTrue(false);
		} catch (ParameterDoesNotExistException e) {
			assertTrue(false);
		}
	}
}
